@@ -33,13 +33,16 @@ import Network.HTTP.Directory
 
 import SimpleCmdArgs
 
-import Common (commonBuildQueryOptions, getBuildState)
+import Common (commonBuildQueryOptions, getBuildState, Limit, lookupArch,
+               maybeLimit, defaultBuildsLimit)
+import Utils (buildlogUrlfromTaskId)
 
 -- FIXME split off arch suffix
 -- FIXME show build duration
 -- FIXME allow buildid
-buildlogSizesCmd :: String -> IO ()
-buildlogSizesCmd nvrpat = do
+-- FIXME only supports Fedora koji
+buildlogSizesCmd :: Maybe Limit -> String -> IO ()
+buildlogSizesCmd mlimit nvrpat = do
   if all isDigit nvrpat -- taskid
     then buildlogSizes (read nvrpat)
     else do -- find builds
@@ -49,10 +52,11 @@ buildlogSizesCmd nvrpat = do
           else nvrpat
     results <- listBuilds fedoraKojiHub
                [("pattern", ValueString pat),
-                commonBuildQueryOptions 5]
+                commonBuildQueryOptions $
+                maybeLimit defaultBuildsLimit mlimit]
     if null results
       then if '*' `notElem` pat
-           then buildlogSizesCmd $ nvrpat ++ "*"
+           then buildlogSizesCmd mlimit $ nvrpat ++ "*"
            else putStrLn $ "no NVRs found for pattern: " ++ pat
       else mapM_ getResult results
   where
@@ -92,7 +96,7 @@ nvrBuildlogSizes nvr = do
 
 buildlogSizes :: Int -> IO ()
 buildlogSizes tid = do
-  children <- sortOn (\t -> lookupStruct "arch" t :: Maybe String) <$>
+  children <- sortOn lookupArch <$>
               getTaskChildren fedoraKojiHub tid True
   mapM_ buildlogSize children
 
@@ -104,10 +108,7 @@ buildlogSize child = do
       whenJust (lookupStruct "arch" child) $
         doGetBuildlogSize buildlog
       where
-        buildlog = "https://kojipkgs.fedoraproject.org/work/tasks" +/+ lastFew +/+ show tid +/+ "build.log"
-        lastFew =
-          let few = dropWhile (== '0') $ drop 4 (show tid) in
-            if null few then "0" else few
+        buildlog = buildlogUrlfromTaskId tid
 
 doGetBuildlogSize :: String -> String -> IO ()
 doGetBuildlogSize buildlog arch = do
@@ -119,9 +120,9 @@ doGetBuildlogSize buildlog arch = do
 #if !MIN_VERSION_http_directory(0,1,9)
 infixr 5 +/+
 (+/+) :: String -> String -> String
-"" +/+ s = s
-s +/+ "" = s
-s +/+ t | last s == '/' = init s +/+ t
-        | head t == '/' = s +/+ tail t
-s +/+ t = s ++ "/" ++ t
+s +/+ t =
+  case (s,t) of
+    ("",_) -> t
+    (_,"") -> s
+    (_,_) -> dropWhileEnd (== '/') s ++ '/' : dropWhile (== '/') t
 #endif
