@@ -9,7 +9,8 @@ module Builds (
   parseBuildState',
   fedoraKojiHub,
   kojiBuildTypes,
-  latestCmd
+  latestCmd,
+  taggedCmd
   )
 where
 
@@ -23,6 +24,7 @@ import Data.Time.Clock
 import Data.Time.LocalTime
 import Distribution.Koji
 import Distribution.Koji.API
+import Safe (headMay)
 import SimpleCmd
 import Text.Pretty.Simple
 
@@ -33,6 +35,7 @@ import Time
 import User
 import Utils (buildOutputURL)
 
+-- FIXME NonEmptyString's
 data BuildReq = BuildBuild String | BuildPackage String
               | BuildQuery | BuildPattern String
   deriving Eq
@@ -45,13 +48,13 @@ capitalize :: String -> String
 capitalize "" = ""
 capitalize (h:t) = toUpper h : t
 
-data Details = DetailDefault | Detailed | DetailedTasks
+data Details = Detailed | DetailedTasks
   deriving Eq
 
-buildsCmd :: Maybe String -> Maybe UserOpt -> Int -> [BuildState]
-          -> Maybe Tasks.BeforeAfter -> Maybe String -> Details
-          -> Maybe Tasks.Select -> Bool -> BuildReq -> IO ()
-buildsCmd mhub museropt limit !states mdate mtype details minstall debug buildreq = do
+buildsCmd :: Maybe String -> Maybe UserOpt -> Maybe Limit -> [BuildState]
+          -> Maybe Tasks.BeforeAfter -> Maybe String -> Maybe Details -> Bool
+          -> Maybe Select -> Bool -> BuildReq -> IO ()
+buildsCmd mhub museropt mlimit !states mdate mtype mdetails tail' minstall debug buildreq = do
   when (hub /= fedoraKojiHub && museropt == Just UserSelf) $
     error' "--mine currently only works with Fedora Koji: use --user instead"
   tz <- getCurrentTimeZone
@@ -63,9 +66,9 @@ buildsCmd mhub museropt limit !states mdate mtype details minstall debug buildre
                     then InfoID (read bld)
                     else InfoString bld
       mbld <- getBuild hub bldinfo
-      whenJust (mbld >>= maybeBuildResult) $ printBuild hub tz details debug minstall
+      whenJust (mbld >>= maybeBuildResult) $ printBuild hub tz mdetails tail' debug minstall
     BuildPackage pkg -> do
-      when (head pkg == '-') $
+      when (headMay pkg == Just '-') $
         error' $ "bad combination: not a package: " ++ pkg
       when (isJust mdate) $
         error' "cannot use --package together with timedate"
@@ -75,21 +78,24 @@ buildsCmd mhub museropt limit !states mdate mtype details minstall debug buildre
         Just pkgid -> do
           query <- setupQuery
           let fullquery = [("packageID", ValueInt pkgid),
-                          commonBuildQueryOptions limit] ++ query
+                           commonBuildQueryOptions $
+                           maybeLimit defaultBuildsLimit mlimit]
+                          ++ query
           when debug $ print fullquery
           builds <- listBuilds hub fullquery
           when debug $ mapM_ pPrintCompact builds
-          if details /= DetailDefault || length builds == 1
-            then mapM_ (printBuild hub tz details debug minstall) $ mapMaybe maybeBuildResult builds
+          if isJust mdetails || length builds == 1
+            then mapM_ (printBuild hub tz mdetails tail' debug minstall) $ mapMaybe maybeBuildResult builds
             else mapM_ putStrLn $ mapMaybe (shortBuildResult tz) builds
     _ -> do
       query <- setupQuery
-      let fullquery = query ++ [commonBuildQueryOptions limit]
+      let fullquery = query ++ [commonBuildQueryOptions $
+                                maybeLimit defaultBuildsLimit mlimit]
       when debug $ print fullquery
       builds <- listBuilds hub fullquery
       when debug $ mapM_ pPrintCompact builds
-      if details /= DetailDefault || length builds == 1
-        then mapM_ (printBuild hub tz details debug minstall) $ mapMaybe maybeBuildResult builds
+      if isJust mdetails || length builds == 1
+        then mapM_ (printBuild hub tz mdetails tail' debug minstall) $ mapMaybe maybeBuildResult builds
         else mapM_ putStrLn $ mapMaybe (shortBuildResult tz) builds
   where
     hub = maybe fedoraKojiHub hubURL mhub
@@ -99,7 +105,7 @@ buildsCmd mhub museropt limit !states mdate mtype details minstall debug buildre
       nvr <- lookupStruct "nvr" bld
       state <- readBuildState <$> lookupStruct "state" bld
       let date =
-            case lookupBuildTimes bld of
+            case lookupStartEndTimes bld of
               Nothing -> ""
               Just (start,mend) ->
                 compactZonedTime tz $ fromMaybe start mend
@@ -115,14 +121,21 @@ buildsCmd mhub museropt limit !states mdate mtype details minstall debug buildre
       whenJust mdatestring $ \date ->
         putStrLn $ maybe "" show mdate +-+ date
       mowner <- maybeGetKojiUser hub museropt
+      pat <-
+        case buildreq of
+          BuildPattern pat -> do
+            -- FIXME remove after moving to newer koji-hs
+            -- rpmfusion koji still doesn't support patterns (2024-09-20)
+            when ("rpmfusion" `isInfixOf` hub) $
+              error' "cannot use pattern with this kojihub"
+            return [("pattern", ValueString pat)]
+          _ -> return []
       return $
         [("complete" ++ (capitalize . show) date, ValueString datestring) | Just date <- [mdate], Just datestring <- [mdatestring]]
         ++ [("userID", ValueInt (getID owner)) | Just owner <- [mowner]]
         ++ [("state", ValueArray (map buildStateToValue states)) | notNull states]
         ++ [("type", ValueString typ) | Just typ <- [mtype]]
-        ++ case buildreq of
-             BuildPattern pat -> [("pattern", ValueString pat)]
-             _ -> []
+        ++ pat
 
     dateString :: Tasks.BeforeAfter -> String
     -- make time refer to past not future
@@ -157,23 +170,25 @@ data BuildResult =
                _buildId :: Int,
                mbuildTaskId :: Maybe Int,
                _buildStartTime :: UTCTime,
-               mbuildEndTime :: Maybe UTCTime
+               mbuildEndTime :: Maybe UTCTime,
+               _buildOwner :: String
               }
 
 maybeBuildResult :: Struct -> Maybe BuildResult
 maybeBuildResult st = do
-  (start,mend) <- lookupBuildTimes st
+  (start,mend) <- lookupStartEndTimes st
   buildid <- lookupStruct "build_id" st
   -- buildContainer has no task_id
   let mtaskid = lookupStruct "task_id" st
   state <- getBuildState st
   nvr <- lookupStruct "nvr" st >>= maybeNVR
+  owner <- lookupStruct "owner_name" st
   return $
-    BuildResult nvr state buildid mtaskid start mend
+    BuildResult nvr state buildid mtaskid start mend owner
 
-printBuild :: String -> TimeZone -> Details -> Bool -> Maybe Tasks.Select
-           -> BuildResult -> IO ()
-printBuild hub tz details debug minstall build = do
+printBuild :: String -> TimeZone -> Maybe Details -> Bool -> Bool
+           -> Maybe Select -> BuildResult -> IO ()
+printBuild hub tz mdetails tail' debug minstall build = do
   putStrLn ""
   let mendtime = mbuildEndTime build
   time <- maybe getCurrentTime return mendtime
@@ -181,24 +196,28 @@ printBuild hub tz details debug minstall build = do
   when (buildState build == BuildComplete) $
     putStrLn $ buildOutputURL hub $ buildNVR build
   whenJust (mbuildTaskId build) $ \taskid -> do
-    when (details == DetailedTasks) $ do
+    if mdetails == Just DetailedTasks
+      then do
       putStrLn ""
-      Tasks.tasksCmd (Just hub) (Tasks.QueryOpts Nothing 7 [] [] Nothing Nothing False Nothing) False False False Nothing (Tasks.Parent taskid)
+      Tasks.tasksCmd (Just hub) Tasks.emptyQueryOpts Nothing tail' Tasks.BuildLog Nothing minstall (Tasks.ChildrenOf taskid)
+      else
+      when tail' $
+      void $ cmdBool "koji" ["watch-task", show taskid]
     whenJust minstall $ \installopts -> do
       putStrLn ""
-      installCmd False debug No (Just hub) Nothing False False False Nothing [] Nothing Nothing installopts Nothing ReqNVR [showNVR (buildNVR build)]
+      installCmd False debug No (Just hub) Nothing False False False Nothing False [] Nothing Nothing installopts Nothing (Right ReqNVR) [showNVR (buildNVR build)]
 
 formatBuildResult :: String -> Bool -> TimeZone -> BuildResult -> [String]
-formatBuildResult hub ended tz (BuildResult nvr state buildid mtaskid start mendtime) =
-  [ showNVR nvr +-+ show state
+formatBuildResult hub ended tz (BuildResult nvr state buildid mtaskid start mendtime owner) =
+  [ showNVR nvr +-+ show state +-+ '(' : owner ++ ")"
   , buildinfoUrl hub buildid]
   ++ [Tasks.taskinfoUrl hub taskid | Just taskid <- [mtaskid]]
-  ++ [formatLocalTime True tz start]
+  ++ [formatLocalTime StartEvent tz start]
   ++
   case mendtime of
     Nothing -> []
     Just end ->
-      [formatLocalTime False tz end | ended]
+      [formatLocalTime CompletionEvent tz end | ended]
 #if MIN_VERSION_time(1,9,1)
       ++
       let dur = diffUTCTime end start
@@ -213,7 +232,10 @@ buildStateToValue = ValueInt . fromEnum
 parseBuildState' :: String -> BuildState
 parseBuildState' s =
   case lower s of
+    "build" -> BuildBuilding
     "building" -> BuildBuilding
+    "open" -> BuildBuilding
+    "run" -> BuildBuilding
     "complete" -> BuildComplete
     "deleted" -> BuildDeleted
     "fail" -> BuildFailed
@@ -221,7 +243,7 @@ parseBuildState' s =
     "cancel" -> BuildCanceled
     "canceled" -> BuildCanceled
     _ -> error' $! "unknown build state: " ++ s ++
-         "\nknown states are: building, complete, deleted, failed, canceled"
+         "\nknown states are: build(ing), complete, deleted, fail(ed), cancel(ed)"
 
 kojiBuildTypes :: [String]
 kojiBuildTypes = ["all", "image", "maven", "module", "rpm", "win"]
@@ -232,4 +254,21 @@ latestCmd mhub debug tag pkg = do
   mbld <- kojiLatestBuild hub tag pkg
   when debug $ print mbld
   tz <- getCurrentTimeZone
-  whenJust (mbld >>= maybeBuildResult) $ printBuild hub tz Detailed debug Nothing
+  whenJust (mbld >>= maybeBuildResult) $ printBuild hub tz (Just Detailed) False debug Nothing
+
+-- FIXME select packages and rpms
+taggedCmd :: Maybe String -> Bool -> String -> IO ()
+taggedCmd mhub install tag = do
+  let hub = maybe fedoraKojiHub hubURL mhub
+  kblds <- kojiListTaggedBuilds hub install tag
+--  when debug $ mapM_ print blds
+--  tz <- getCurrentTimeZone
+  -- forM_ (mapMaybe maybeBuildResult blds) $
+  --   printBuild hub tz (Just Detailed) False debug Nothing
+  mapM_ putKojiBuild kblds
+  when install $
+    installCmd False False No (Just hub) Nothing False False False Nothing False [] Nothing Nothing selectDefault Nothing (Right ReqNVR) $ map kbNvr kblds
+  where
+    putKojiBuild :: KojiBuild -> IO ()
+    putKojiBuild (KojiBuild _ _ owner nvr) =
+      putStrLn $ nvr +-+ owner
