@@ -6,11 +6,13 @@ module Main (main) where
 
 import Data.Char (isDigit)
 import Data.List.Extra
+import SelectRPMs (existingStrategyOption, pkgMgrOpt, selectRpmsOptions)
 import SimpleCmd
 import SimpleCmdArgs
 
 import Builds
 import BuildlogSizes
+import Common (Limit(..), Natural, defaultBuildsLimit, defaultTasksLimit)
 import User
 import Install
 import qualified Paths_koji_tool
@@ -29,14 +31,15 @@ main =
       buildsCmd
       <$> hubOpt
       <*> optional (userOpt False)
-      <*> (flagWith' 1 'L' "latest" "Latest build" <|>
-           optionalWith auto 'l' "limit" "INT" "Maximum number of builds to show [default: 10]" 10)
+      <*> optional (limitOpt "build" defaultBuildsLimit)
       <*> many (parseBuildState' <$> strOptionWith 's' "state" "STATE" "Filter builds by state (building,complete,deleted,fail(ed),cancel(ed)")
       <*> optional (Before <$> strOptionWith 'B' "before" "TIMESTAMP" "Builds completed before timedate [default: now]" <|>
                     After <$> strOptionWith 'F' "from" "TIMESTAMP" "Builds completed after timedate")
-      <*> (fmap normalizeBuildType <$> optional (strOptionWith 'T' "type" "TYPE" ("Select builds by type: " ++ intercalate "," kojiBuildTypes)))
-      <*> (flagWith' Detailed 'd' "details" "Show more build details" <|>
-           flagWith DetailDefault DetailedTasks 't' "tasks" "Show details and tasks")
+      <*> (fmap normalizeBuildType <$> optional (strOptionLongWith "type" "TYPE" ("Select builds by type: " ++ intercalate "," kojiBuildTypes)))
+      <*> optional
+      (flagWith' Builds.Detailed 'd' "details" "Show more build details" <|>
+       flagWith' Builds.DetailedTasks 't' "tasks" "Show details and tasks")
+      <*> tailOpt
       <*> optional (installArgs <$> strOptionWith 'i' "install" "INSTALLOPTS" "Install the package with 'install' options")
       <*> switchWith 'D' "debug" "Pretty-print raw XML result"
       <*> (BuildBuild <$> strOptionWith 'b' "build" "NVR/BUILDID" "Show build" <|>
@@ -49,14 +52,16 @@ main =
       tasksCmd
       <$> hubOpt
       <*> queryOpts False "buildArch"
-      <*> switchWith 'd' "details" "Show more details of builds"
-      -- FIXME error if integer (eg mistakenly taskid)
-      <*> switchWith 'T' "tail" "Fetch the tail of build.log"
-      <*> switchLongWith "hw-info" "Fetch hw_info.log"
+      <*> optional
+      (flagWith' Tasks.Detailed 'd' "details" "Show more task details" <|>
+       flagWith' Tasks.Concise 'c' "concise" "Compact task output")
+      <*> tailOpt
+      <*> (flagLongWith' HWInfo "hw-info" "Fetch hw_info.log" <|>
+           flagLongWith BuildLog RootLog "rootlog" "Fetch root.log")
       <*> optional (strOptionWith 'g' "grep" "STRING" "Filter matching log lines")
-      -- -- FIXME any way to pass --help to install?
-      -- <*> optional (installArgs <$> strOptionWith 'i' "install" "INSTALLOPTS" "Install the package with 'install' options")
+      <*> optional (installArgs <$> strOptionWith 'i' "install" "INSTALLOPTS" "Install the package with 'install' options")
       <*> taskReqOpt
+
     , Subcommand "latest"
       "Query latest Koji build for tag" $
       latestCmd
@@ -75,36 +80,46 @@ main =
       <*> hubOpt
       <*> optional (strOptionWith 'P' "packages-url" "URL"
                     "KojiFiles packages url [default: Fedora]")
-      <*> switchWith 'l' "list" "List builds"
+      <*> switchWith 'l' "list" "List RPMs"
       <*> switchWith 'L' "latest" "Latest build"
       <*> switchWith 't' "check-remote-time" "Check remote rpm timestamps"
       <*> optional pkgMgrOpt
+      <*> switchLongWith "allow-erasing" "Use dnf --allowerasing"
       <*> many archOpt
-      <*> optional existingOpt
+      <*> optional existingStrategyOption
       <*> optional (strOptionWith 'b' "prefix" "SUBPKGPREFIX" "Prefix to use for subpackages [default: base package]")
-      <*> selectOpt
+      <*> selectRpmsOptions
       <*> optional disttagOpt
-      <*> (flagWith' ReqNVR 'R' "nvr" "Give an N-V-R instead of package name" <|>
-           flagWith ReqName ReqNV 'V' "nv" "Give an N-V instead of package name")
-      <*> some (strArg "PKG|NVR|TASKID...")
+      <*> (flagLongWith' (Left ()) "tagged" "Give a tag instead of package name" <|>
+           flagWith' (Right ReqNVR) 'R' "nvr" "Give an N-V-R instead of package name" <|>
+           flagWith (Right ReqName) (Right ReqNV) 'V' "nv" "Give an N-V instead of (default) package name")
+      <*> some (strArg "PKG|NVR|TASKID|TAG...")
 
     , Subcommand "progress"
       "Track running Koji tasks by buildlog size" $
       progressCmd
-      <$> switchWith 'm' "modules" "Track module builds"
+      <$> switchLongWith "modules" "Track module builds"
       <*> queryOpts True "build"
       <*> taskReqOpt
 
     , Subcommand "buildlog-sizes" "Show buildlog sizes for nvr patterns" $
-      buildlogSizesCmd <$> strArg "NVRPATTERN|PKG|TASKID"
+      buildlogSizesCmd
+      <$> optional (limitOpt "build" defaultBuildsLimit)
+      <*> strArg "NVRPATTERN|PKG|TASKID"
 
     , Subcommand "find"
       ("Simple quick common queries using words like: [" ++
-       intercalate ", " (wordsList head) ++ "]") $
+       intercalate ", " wordsListHeads ++ "]") $
       findCmd
       <$> hubOpt
-      <*> switchWith 'D' "debug" "Debug output including XML results"
       <*> many (strArg "PHRASE")
+
+    , Subcommand "tagged"
+      "List builds for tag" $
+      taggedCmd
+      <$> hubOpt
+      <*> switchWith 'i' "install" "Install packages"
+      <*> strArg "TAG"
     ]
   where
     hubOpt = optional (strOptionWith 'H' "hub" "HUB"
@@ -117,16 +132,6 @@ main =
       User <$> strOptionWith 'u' "user" "USER" "Koji user"
       <|> if mine then pure UserSelf else flagWith' UserSelf 'M' "mine" "Your tasks (krb fasid)"
 
-    selectOpt :: Parser Select
-    selectOpt =
-      flagLongWith' All "all" "all subpackages [default if not installed]" <|>
-      flagLongWith' Ask "ask" "ask for each subpackage" <|>
-      PkgsReq
-      <$> many (strOptionWith 'p' "package" "SUBPKG" "select subpackage (glob) matches")
-      <*> many (strOptionWith 'e' "except" "SUBPKG" "select subpackages not matching (glob)")
-      <*> many (strOptionWith 'x' "exclude" "SUBPKG" "deselect subpackage (glob): overrides -p and -e")
-      <*> many (strOptionWith 'i' "include" "SUBPKG" "additional subpackage (glob) to install: overrides -x")
-
     disttagOpt :: Parser String
     disttagOpt = startingDot <$>
                  strOptionWith 'd' "disttag" "DISTTAG"
@@ -153,18 +158,6 @@ main =
     readTaskReq cs =
       Just $ if all isDigit cs then Task (read cs) else Package cs
 
-    pkgMgrOpt :: Parser PkgMgr
-    pkgMgrOpt =
-      flagLongWith' RPM "rpm" "Use rpm instead of dnf" <|>
-      flagLongWith' OSTREE "rpm-ostree" "Use rpm-ostree instead of dnf" <|>
-      flagLongWith' DNF5 "dnf5" "Use dnf5 to install" <|>
-      flagLongWith' DNF3 "dnf3" "Use dnf-3 to install [default dnf unless ostree]"
-
-    existingOpt :: Parser ExistingStrategy
-    existingOpt =
-      flagWith' ExistingNoReinstall 'N' "no-reinstall" "Do not reinstall existing NVRs" <|>
-      flagWith' ExistingSkip 'S' "skip-existing" "Ignore already installed subpackages (implies --no-reinstall)"
-
     -- FIXME check valid arch (eg i686 not i386)
     archOpt = strOptionWith 'a' "arch" "ARCH" "Task arch"
 
@@ -172,8 +165,7 @@ main =
     queryOpts mine defaultMethod =
       QueryOpts
       <$> optional (userOpt mine)
-      <*> (flagWith' 1 'L' "latest" "Latest build or task" <|>
-           optionalWith auto 'l' "limit" "INT" "Maximum number of tasks to show [default: 10]" 10)
+      <*> optional (limitOpt "build/task" defaultTasksLimit)
       <*> many (fmap parseTaskState' $! strOptionWith 's' "state" "STATE" "Filter tasks by state (open,close(d),cancel(ed),fail(ed),assigned,free)")
       <*> many archOpt
       <*> optional (Before <$> strOptionWith 'B' "before" "TIMESTAMP" "Tasks completed before timedate [default: now]" <|>
@@ -183,8 +175,19 @@ main =
       <*> optional (TaskPackage <$> strOptionWith 'P' "only-package" "PKG" "Filter task results to specified package"
                    <|> TaskNVR <$> strOptionWith 'N' "only-nvr" "PREFIX" "Filter task results by NVR prefix")
 
+    -- FIXME error if integer (eg mistakenly taskid) instead of string
     taskReqOpt =
       Build <$> strOptionWith 'b' "build" "BUILD" "List child tasks of build"
       <|> Pattern <$> strOptionWith 'p' "pattern" "NVRPAT" "Build tasks of matching pattern"
+      <|> ChildrenOf <$> optionLongWith auto "children" "TASKID" "Children tasks of parent"
+      <|> ParentOf <$> optionLongWith auto "parent" "TASKID" "Parent of task"
       <|> argumentWith (maybeReader readTaskReq) "PACKAGE|TASKID"
       <|> pure TaskQuery
+
+    limitOpt :: String -> Natural -> Parser Limit
+    limitOpt tgt dflt =
+      flagWith' (Limit 1) 'L' "latest" ("Latest" +-+ tgt) <|>
+      flagWith' Nolimit 'U' "unlimited" "No limit on number of results" <|>
+      Limit <$> optionWith auto 'l' "limit" "INT" ("Maximum number of" +-+ tgt ++ "s to show [default:" +-+ show dflt ++ "]")
+
+    tailOpt = switchWith 'T' "tail" "Fetch the tail of build.log"
