@@ -33,9 +33,10 @@ import Data.Time (diffUTCTime, getCurrentTime, getCurrentTimeZone,
 import Data.Text (Text)
 import qualified Data.Text as T
 import Distribution.Koji
+import Safe (maximumMay)
 import SimpleCmd
-import System.FilePath ((</>))
 
+import Common (lookupArch)
 import Tasks
 import Time
 import Utils
@@ -44,7 +45,6 @@ import Utils
 -- FIXME if failure and no more, then stop
 -- FIXME catch HTTP exception for connection timeout
 -- FIXME pick up new user builds (if none specified)
--- FIXME noarch tasks shown with build arch
 progressCmd :: Bool -> QueryOpts -> TaskReq -> IO ()
 progressCmd modules queryopts@QueryOpts{..} taskreq = do
   tasks <- do
@@ -113,9 +113,10 @@ data BuildTask =
 
 initialBuildTask :: Struct -> IO BuildTask
 initialBuildTask taskinfo = do
-  let tid = case lookupStruct "id" taskinfo of
-              Just tid' -> TaskId tid'
-              Nothing -> error' $ "no taskid found for:" ++ show taskinfo
+  let tid =
+        case lookupStruct "id" taskinfo of
+          Just tid' -> TaskId tid'
+          Nothing -> error' $ "no taskid found for:" ++ show taskinfo
       parent =
         case lookupStruct "method" taskinfo :: Maybe String of
           Nothing -> error' $ "no method found for " ++ displayID tid
@@ -130,7 +131,7 @@ initialBuildTask taskinfo = do
               -- https://koji.fedoraproject.org/koji/taskinfo?taskID=104446426
               -- https://kojipkgs.fedoraproject.org//work/tasks/6426/104446426/
               _ -> error' $ "unsupported method: " ++ method
-  children <- sortOn (\t -> lookupStruct "arch" t :: Maybe String) <$>
+  children <- sortOn lookupArch <$>
                       kojiGetTaskChildren fedoraKojiHub parent True
   let start =
         case lookupTime CreateEvent taskinfo of
@@ -197,10 +198,8 @@ loopBuildTasks debug tz bts = do
           printLogStatuses header tz statuses
           let news = map (\(task',(s,t),_) -> TaskInfoStatus task' (mkTaskStatus s t (getTaskState task'))) statuses
               (open,closed) = partition (\tis -> getTaskState (taskInfo tis) `elem` map Just openTaskStates) news
-              mlargest = if not (any (\tis -> lookupStruct "method" (taskInfo tis) /= Just ("buildSRPMFromSCM" :: String)) closed)
-                         then Nothing
-                         else Just $ maximum $ mapMaybe (\t -> taskStatus t >>= tstLog <&> logSize) closed
-              mbiggest = max mlargest msize
+              closedbuilds = filter (\tis -> lookupStruct "method" (taskInfo tis) == Just ("buildArch" :: String)) closed
+              mbiggest = maximumMay $ maybeToList msize ++ mapMaybe (\t -> taskStatus t >>= tstLog <&> logSize) closedbuilds
           if null open
             then runProgress (BuildTask tid start mend mbiggest [])
             else return $ BuildTask tid start mend mbiggest open
@@ -212,6 +211,8 @@ loopBuildTasks debug tz bts = do
 
 buildlogSize :: Bool -> Int -> TaskInfoStatus -> IO TaskInfoStatuses
 buildlogSize debug n (TaskInfoStatus task moldstatus) = do
+  let tid = fromJust (lookupStruct "id" task)
+      buildlog = buildlogUrlfromTaskId tid
   when debug $ putStrLn buildlog
   exists <- if isJust moldstatus
             then return True
@@ -221,7 +222,7 @@ buildlogSize debug n (TaskInfoStatus task moldstatus) = do
   (msize,mtime) <- if exists
                    then httpFileSizeTime' buildlog
                    else return (Nothing,Nothing)
-  when debug $ print (mtime,moldstatus)
+  when debug $ print (msize,mtime,moldstatus)
   if (mtime == fmap logTime moldlog || isNothing mtime) && n < 5
     then buildlogSize debug (n+1) (TaskInfoStatus task moldstatus)
     else
@@ -229,12 +230,6 @@ buildlogSize debug n (TaskInfoStatus task moldstatus) = do
             (fromInteger <$> msize, mtime),
             moldstatus)
   where
-    tid = show $ fromJust (readID' task)
-    buildlog = "https://kojipkgs.fedoraproject.org/work/tasks" </> lastFew </> tid </> "build.log"
-    lastFew =
-      let few = dropWhile (== '0') $ takeEnd 4 tid in
-        if null few then "0" else few
-
     moldlog = moldstatus >>= tstLog :: Maybe LogStatus
 
     waitDelay ::  IO ()
@@ -271,7 +266,7 @@ threadDelaySeconds m =
   threadDelay (fromEnum (fromIntegral m :: Micro))
 
 
-data TaskOutput = TaskOut {_outArch :: Text,
+data TaskOutput = TaskOut {_outArch :: String,
                            moutSize :: Maybe Int,
                            moutSizeStep :: Maybe Int,
                            outSizeChanged :: Bool,
@@ -287,15 +282,15 @@ printLogStatuses :: IO () -> TimeZone -> [TaskInfoStatuses] -> IO ()
 printLogStatuses header tz tss =
   let (mxsi, mxsp, taskoutputs) = (formatSize . map taskOutput) tss
   in
-    unless (null taskoutputs) $ do
+    unless (null taskoutputs) $
     when (any (\t -> outTimeChanged t || outSizeChanged t || outStateChanged t) taskoutputs) $ do
-      header
-      mapM_ (printTaskOut mxsi mxsp) taskoutputs
-      putChar '\n'
+    header
+    mapM_ (printTaskOut mxsi mxsp) taskoutputs
+    putChar '\n'
   where
     printTaskOut :: Int64 -> Int64 -> TaskOutput -> IO ()
     printTaskOut maxsize maxspd (TaskOut arch msize msizediff _sizechanged mtime mtimediff _timechanged state _statechanged mthd mduration) =
-      fprintLn (rpadded 8 ' ' stext %
+      fprintLn (rpadded 8 ' ' string %
                 lpadded (max 6 (maxsize+2)) ' ' (optioned commas) % "kB" %
                 " " %
                 optioned (parenthesised string % " ") %
@@ -339,7 +334,7 @@ printLogStatuses header tz tss =
           mstate = getTaskState task
       in
         let method = maybeVal "method not found" (lookupStruct "method") task :: Text
-            arch = maybeVal "arch not found" (lookupStruct "arch") task :: Text
+            arch = maybeVal "arch not found" lookupArch task
             sizediff = liftM2 (-) size oldsize
             timediff = if time == oldtime
                        then Nothing
@@ -358,6 +353,7 @@ kojiListBuildTasks muser = do
   user <- case muser of
             Just user -> return user
             Nothing -> do
+              -- FIXME test for klist
               mfasid <- (removeSuffix "@FEDORAPROJECT.ORG" <$>) . find ("@FEDORAPROJECT.ORG" `isSuffixOf`) . words <$> cmd "klist" ["-l"]
               case mfasid of
                 Just fas -> return fas
