@@ -7,16 +7,19 @@ module Tasks (
   TaskReq(..),
   BeforeAfter(..),
   QueryOpts(..),
+  emptyQueryOpts,
+  Details(..),
+  LogFile(..),
   tasksCmd,
   getTasks,
   parseTaskState',
   kojiMethods,
   fedoraKojiHub,
   taskinfoUrl,
-  Select(PkgsReq)
   )
 where
 
+import Control.Applicative ((<|>))
 import Control.Monad.Extra
 import qualified Data.ByteString.Lazy.UTF8 as U
 import Data.Char (isDigit, toUpper)
@@ -33,6 +36,8 @@ import Distribution.Koji.API
 import Formatting hiding (now)
 import Network.HTTP.Directory
 import Network.HTTP.Simple
+import Network.HTTP.Types (hAccept)
+import Safe (headMay, lastMay)
 import SimpleCmd
 import System.FilePath
 import Text.Pretty.Simple
@@ -43,7 +48,8 @@ import Time
 import User
 import Utils
 
-data TaskReq = Task Int | Parent Int | Build String | Package String
+data TaskReq = Task Int | ChildrenOf Int | ParentOf Int
+             | Build String | Package String
              | TaskQuery | Pattern String
 
 data TaskFilter = TaskPackage String | TaskNVR String
@@ -62,16 +68,18 @@ data TaskResult =
   TaskResult {taskPackage :: Either String NVR,
               taskArch :: String,
               taskMethod :: String,
-              _taskState :: TaskState,
+              taskState :: TaskState,
               _mtaskParent :: Maybe Int,
               taskId :: Int,
+              _taskCreateTime :: UTCTime,
               _mtaskStartTime :: Maybe UTCTime,
-              mtaskEndTime :: Maybe UTCTime
+              mtaskEndTime :: Maybe UTCTime,
+              _mtaskOwner :: Maybe String
              }
 
 data QueryOpts = QueryOpts {
   qmUserOpt :: Maybe UserOpt,
-  qLimit :: Int,
+  qLimit :: Maybe Limit,
   qStates :: ![TaskState],
   qArchs :: ![String],
   qmDate :: Maybe BeforeAfter,
@@ -79,24 +87,34 @@ data QueryOpts = QueryOpts {
   qDebug :: Bool,
   qmFilter :: Maybe TaskFilter}
 
+emptyQueryOpts :: QueryOpts
+emptyQueryOpts =
+  QueryOpts Nothing Nothing [] [] Nothing Nothing False Nothing
+
+data Details = Detailed | Concise
+  deriving Eq
+
+-- FIXME 'koji-tool tasks <taskid> -s fail -m buildarch' should not print build task
 -- FIXME short output option
 -- --sibling
 -- FIXME --tail-size option (eg more that 4000B)
 -- FIXME --output-fields
 -- FIXME default to 'build' for install or try 'build' after 'buildarch'?
--- FIXME parent tasks need not have limit
 -- FIXME `-# 2` etc to select second result
-tasksCmd :: Maybe String -> QueryOpts -> Bool -> Bool -> Bool -> Maybe String
-         -> TaskReq -> IO ()
-tasksCmd mhub queryopts@QueryOpts{..} details tail' hwinfo mgrep taskreq = do
+tasksCmd :: Maybe String -> QueryOpts -> Maybe Details -> Bool -> LogFile
+         -> Maybe String -> Maybe Select -> TaskReq -> IO ()
+tasksCmd mhub queryopts@QueryOpts{..} mdetails tail' logfile mgrep minstall taskreq = do
   when (hub /= fedoraKojiHub && qmUserOpt == Just UserSelf) $
     error' "--mine currently only works with Fedora Koji: use --user instead"
   tz <- getCurrentTimeZone
   tasks <- getTasks tz hub queryopts taskreq
   when qDebug $ mapM_ pPrintCompact tasks
-  let exact = length tasks == 1
-      detailed = details || exact
-  (mapM_ (printTask detailed tz) . filterResults . mapMaybe maybeTaskResult) tasks
+  let details =
+        case mdetails of
+          Nothing ->
+            if length tasks == 1 then Detailed else Concise
+          Just detail -> detail
+  (mapM_ (printTask details tz) . filterResults . mapMaybe maybeTaskResult) tasks
   where
     hub = maybe fedoraKojiHub hubURL mhub
 
@@ -115,39 +133,47 @@ tasksCmd mhub queryopts@QueryOpts{..} details tail' hwinfo mgrep taskreq = do
         isNVR _ (Left _) = False
         isNVR nvr (Right nvr') = nvr `isPrefixOf` showNVR nvr'
 
-    printTask :: Bool -> TimeZone -> TaskResult -> IO ()
-    printTask detailed tz task = do
+    printTask :: Details -> TimeZone -> TaskResult -> IO ()
+    printTask details tz task = do
       let mendtime = mtaskEndTime task
       mtime <- if isNothing  mendtime
                  then Just <$> getCurrentTime
                  else return Nothing
-      if detailed
+      if details == Detailed
         then do
         putStrLn ""
-        -- FIX for parent/build method show children (like we do with taskid)
+        -- FIXME for parent/build method show children (like we do with taskid)
         (mapM_ putStrLn . formatTaskResult hub mtime tz) task
         if taskMethod task == "build"
           then do
-          getTasks tz hub queryopts (Parent $ taskId task) >>=
-            mapM_ (printTask detailed tz) . mapMaybe maybeTaskResult
-          else buildlogSize qDebug tail' hwinfo mgrep hub task
+          when (mdetails == Just Detailed) $
+            getTasks tz hub queryopts (ChildrenOf $ taskId task) >>=
+            -- FIXME interleave newlines
+            mapM_ (printTask details tz) . mapMaybe maybeTaskResult
+          else buildlogSize qDebug tz tail' logfile mgrep hub task
         else do
         (putStrLn . compactTaskResult hub tz) task
-        when (tail' || hwinfo || isJust mgrep) $
-          buildlogSize qDebug tail' hwinfo mgrep hub task
+        when (tail' || logfile /= BuildLog || isJust mgrep) $
+          buildlogSize qDebug tz tail' logfile mgrep hub task
+      whenJust minstall $ \installopts -> do
+        putStrLn ""
+        installCmd False qDebug No (Just hub) Nothing False False False Nothing False [] Nothing Nothing installopts Nothing (Right ReqName) [show (taskId task)]
 
 maybeTaskResult :: Struct -> Maybe TaskResult
 maybeTaskResult st = do
-  arch <- lookupStruct "arch" st
-  let mstart_time = lookupTime CreateEvent st
-      mend_time = lookupTime CompletionEvent st
+  arch <- lookupArch st
+  let (create,mstart,mend) = lookupTaskTimes st
   taskid <- lookupStruct "id" st
   method <- lookupStruct "method" st
   state <- getTaskState st
   let pkgnvr = kojiTaskRequestNVR st
       mparent' = lookupStruct "parent" st :: Maybe Int
+  -- FIXME filter long names like
+  -- "koschei/koschei-backend01.iad2.fedoraproject.org"
+  -- "bpeck/jenkins-continuous-infra.apps.ci.centos.org"
+      mowner = lookupStruct "owner_name" st
   return $
-    TaskResult pkgnvr arch method state mparent' taskid mstart_time mend_time
+    TaskResult pkgnvr arch method state mparent' taskid create mstart mend mowner
 
 pPrintCompact :: Struct -> IO ()
 pPrintCompact =
@@ -175,11 +201,11 @@ getTasks tz hub queryopts@QueryOpts {..} req =
         error' "cannot use taskid together with filter"
       mtask <- kojiGetTaskInfo hub (TaskId taskid)
       case mtask of
-        Nothing -> error $ "taskid not found: " ++ show taskid
+        Nothing -> error $ "taskid not found:" +-+ show taskid
         Just task -> do
           when qDebug $ pPrintCompact task
           case maybeTaskResult task of
-            Nothing -> error' $ "failed to read task: " ++ show task
+            Nothing -> error' $ "failed to read task:" +-+ show task
             -- FIXME maybe should have way to list parent or children
             Just _res -> return [task]
     Build bld -> do
@@ -191,49 +217,64 @@ getTasks tz hub queryopts@QueryOpts {..} req =
                       getBuild hub (InfoID (read bld))
                  else kojiGetBuildTaskID hub bld
       case mtaskid of
-        Just (TaskId taskid) -> getTasks tz hub queryopts $ Parent taskid
-        Nothing -> error' $ "no taskid found for build " ++ bld
+        Just (TaskId taskid) -> getTasks tz hub queryopts $ ChildrenOf taskid
+        Nothing -> error' $ "no taskid found for build" +-+ bld
     Package pkg -> do
-      when (head pkg == '-') $
-        error' $ "bad combination: not a package " ++ pkg
+      when (headMay pkg == Just '-') $
+        error' $ "bad combination: not a package" +-+ pkg
       when (isJust qmDate || isJust qmFilter) $
         -- FIXME why not?
         error' "cannot use package together with timedate or filter"
       mpkgid <- getPackageID hub pkg
       case mpkgid of
-        Nothing -> error' $ "no package id found for " ++ pkg
+        Nothing -> error' $ "no package id found for" +-+ pkg
         Just pkgid -> do
           builds <- listBuilds hub
                     [("packageID", ValueInt pkgid),
-                     commonBuildQueryOptions qLimit]
+                     commonBuildQueryOptions $
+                     maybeLimit defaultBuildsLimit qLimit]
           fmap concat <$>
             forM builds $ \bld -> do
             let mtaskid = (fmap TaskId . lookupStruct "task_id") bld
             case mtaskid of
               -- FIXME gives too many tasks (parent builds):
-              Just (TaskId taskid) -> getTasks tz hub queryopts $ Parent taskid
+              Just (TaskId taskid) -> getTasks tz hub queryopts {qLimit = Nothing} $ ChildrenOf taskid
               Nothing -> return []
+    ParentOf taskid -> do
+      mtask <- kojiGetTaskInfo hub (TaskId taskid)
+      case mtask of
+        Nothing -> error $ "taskid not found:" +-+ show taskid
+        Just task -> do
+          when qDebug $ pPrintCompact task
+          case lookupStruct "parent" task of
+            Nothing -> error' $ "no parent of" +-+ show taskid
+            Just tid -> getTasks tz hub queryopts {qLimit = Nothing} $ Task tid
     Pattern pat -> do
       let buildquery = [("pattern", ValueString pat),
-                        commonBuildQueryOptions qLimit]
+                        commonBuildQueryOptions $
+                        maybeLimit defaultBuildsLimit qLimit]
       when qDebug $ print buildquery
+      -- FIXME remove after moving to newer koji-hs
+      -- rpmfusion koji still doesn't support patterns (2024-09-20)
+      when ("rpmfusion" `isInfixOf` hub) $
+        error' "cannot use pattern with this kojihub"
       builds <- listBuilds hub buildquery
       when qDebug $ print builds
       fmap concat <$>
         forM builds $ \bld -> do
         let mtaskid = (fmap TaskId . lookupStruct "task_id") bld
         case mtaskid of
-          Just (TaskId taskid) -> getTasks tz hub queryopts $ Parent taskid
+          Just (TaskId taskid) -> getTasks tz hub (queryopts {qLimit = Nothing}) $ ChildrenOf taskid
           Nothing -> return []
     _ -> do
       query <- setupQuery
-      let qopts = commonQueryOptions qLimit "-id"
+      let qopts = commonQueryOptions (maybeLimit defaultTasksLimit qLimit) "-id"
       when qDebug $ print $ query ++ qopts
       listTasks hub query qopts
   where
     setupQuery = do
       case req of
-        Parent parent ->
+        ChildrenOf parent ->
           return $ ("parent", ValueInt parent) : commonParams
         _ -> do
           mdatestring <-
@@ -253,29 +294,24 @@ getTasks tz hub queryopts@QueryOpts {..} req =
           commonParams =
             [("decode", ValueBool True)]
             ++ [("state", ValueArray (map taskStateToValue qStates)) | notNull qStates]
-            ++ [("arch", ValueArray (map (ValueString . kojiArch) qArchs)) | notNull qArchs]
+            ++ [("arch", ValueArray (map (ValueString . kojiTaskArch) qArchs)) | notNull qArchs]
             ++ [("method", ValueString method) | let method = fromMaybe defaultTaskMethod qmMethod]
 
           capitalize :: String -> String
           capitalize "" = ""
           capitalize (h:t) = toUpper h : t
 
-          kojiArch :: String -> String
-          kojiArch "i686" = "i386"
-          kojiArch "armv7hl" = "armhfp"
-          kojiArch a = a
-
           dateString :: BeforeAfter -> String
           -- make time refer to past not future
           dateString beforeAfter =
             let timedate = getTimedate beforeAfter
             in case words timedate of
                  [t] | t `elem` ["hour", "day", "week", "month", "year"] ->
-                       "last " ++ t
+                       "last" +-+ t
                  [t] | t `elem` ["today", "yesterday"] ->
                        t ++ " 00:00"
                  [t] | any (lower t `isPrefixOf`) ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"] ->
-                       "last " ++ t ++ " 00:00"
+                       "last" +-+ t ++ " 00:00"
                  [n,_unit] | all isDigit n -> timedate ++ " ago"
                  _ -> timedate
 
@@ -285,7 +321,7 @@ taskinfoUrl hub tid =
 
 -- FIXME option to hide url (take terminal width into consideration?)
 compactTaskResult :: String -> TimeZone -> TaskResult -> String
-compactTaskResult hub tz (TaskResult pkg arch method state _mparent taskid mstart mend) =
+compactTaskResult hub tz (TaskResult pkg arch method state _mparent taskid _create mstart mend _mowner) =
   let time =
         case mend of
           Just end -> compactZonedTime tz end
@@ -296,10 +332,8 @@ compactTaskResult hub tz (TaskResult pkg arch method state _mparent taskid mstar
      taskinfoUrl hub taskid] ++
     ["(" ++ renderDuration True dur ++ ")" | Just start <- [mstart],  Just end <- [mend], let dur = diffUTCTime end start] ++
     [show state,
-     showPackage pkg ++ if method == "buildArch" then '.' : arch ++ replicate (8 - length arch) ' ' else ' ' : method]
+     showPackage pkg ++ if method == "buildArch" then '.' : arch else ' ' : method]
 
-
--- FIXME show task owner
 formatTaskResult :: String -> Maybe UTCTime -> TimeZone -> TaskResult -> [String]
 formatTaskResult hub
 #if MIN_VERSION_time(1,9,1)
@@ -307,19 +341,18 @@ formatTaskResult hub
 #else
   _mtime
 #endif
-  tz (TaskResult pkg arch method state mparent taskid mstart mend) =
-  [ showPackage pkg ++ (if method == "buildArch" then '.' : arch else ' ' : method) +-+ show state
-  , taskinfoUrl hub taskid +-+ maybe "" (\p -> "(parent: " ++ show p ++ ")") mparent] ++
-  [formatLocalTime True tz start | Just start <- [mstart]] ++
-  [formatLocalTime False tz end | Just end <- [mend]]
+  tz (TaskResult pkg arch method state mparent taskid create mstart mend mowner) =
+  [ showPackage pkg ++ (if method == "buildArch" then '.' : arch else ' ' : method) +-+ show state +-+ maybe "" (\o -> '(' : o ++ ")") mowner
+  , taskinfoUrl hub taskid +-+ maybe "" (\p -> "(parent:" +-+ show p ++ ")") mparent
+  , formatLocalTime CreateEvent tz create] ++
+  [formatLocalTime StartEvent tz start | Just start <- [mstart]] ++
+  [formatLocalTime CompletionEvent tz end | Just end <- [mend]]
 #if MIN_VERSION_time(1,9,1)
-      ++
-    case mtime of
-      Just now ->
-        ["current duration: " ++ renderDuration False dur | Just start <- [mstart], let dur = diffUTCTime now start]
-      Nothing ->
-        ["duration: " ++ renderDuration False dur | Just start <- [mstart], Just end <- [mend], let dur = diffUTCTime end start]
-
+    ++
+    case mtime <|> mend of
+      Just end ->
+        [(if isJust mtime then "current" else "") +-+ "duration:" +-+ renderDuration False dur +-+ '(' : "start delay" +-+ renderDuration False delay ++ ")" | Just start <- [mstart], let dur = diffUTCTime end start, let delay = diffUTCTime start create]
+      Nothing -> []
 #endif
 
 showPackage :: Either String NVR -> String
@@ -343,7 +376,7 @@ parseTaskState' s =
     "assigned" -> TaskAssigned
     "fail" -> TaskFailed
     "failed" -> TaskFailed
-    _ -> error' $! "unknown task state: " ++ s ++
+    _ -> error' $! "unknown task state:" +-+ s ++
          "\nknown states: free, open, closed, canceled, assigned, failed"
 
 data LogFile = BuildLog | RootLog | HWInfo
@@ -354,7 +387,7 @@ data OutputLocation = PackagesOutput | WorkOutput
 outputUrl :: String -> TaskResult -> OutputLocation -> Maybe String
 outputUrl hub task loc =
   case loc of
-    WorkOutput -> Just $ taskOutputUrl task
+    WorkOutput -> Just $ worktaskDirUrlfromTaskID $ taskId task
     PackagesOutput ->
       case taskPackage task of
         Left _ -> Nothing
@@ -364,116 +397,123 @@ outputUrl hub task loc =
 findOutputURL :: String -> TaskResult -> IO (Maybe String)
 findOutputURL hub task =
   case outputUrl hub task PackagesOutput of
-    Just burl -> urlExistsOr (taskOutputUrl task) $
-                 urlExistsOr burl $ return Nothing
-    Nothing -> urlExistsOr (taskOutputUrl task) $ return Nothing
+    Just burl -> urlMayExist (worktaskDirUrlfromTaskID $ taskId task) <||>
+                 urlMayExist burl
+    Nothing -> urlMayExist (worktaskDirUrlfromTaskID $ taskId task)
   where
-    urlExistsOr :: String -> IO (Maybe String) -> IO (Maybe String)
-    urlExistsOr url alt = do
+    urlMayExist :: String -> IO (Maybe String)
+    urlMayExist url = do
       exists <- httpExists' url
-      if exists
-        then return $ Just url
-        else alt
+      return $
+        if exists
+        then Just url
+        else Nothing
 
-taskOutputUrl :: TaskResult -> String
-taskOutputUrl task =
-  "https://kojipkgs.fedoraproject.org/work/tasks" </> lastFew </> tid
-  where
-    tid = show (taskId task)
+    mact1 <||> mact2 = do
+      ma1 <- mact1
+      if isJust ma1
+        then return ma1
+        else mact2
 
-    lastFew =
-      let few = dropWhile (== '0') $ takeEnd 4 tid
-      in if null few then "0" else few
-
-tailLogUrl :: String -> Int -> LogFile -> String
-tailLogUrl hub taskid file =
-  webUrl hub +/+ "getfile?taskID=" ++ show taskid ++ "&name=" ++ logFile file ++ "&offset=-6000"
+-- FIXME moffset not actually used yet
+tailLogUrl :: String -> Int -> LogFile -> Maybe Int -> String
+tailLogUrl hub taskid file moffset =
+  webUrl hub +/+ "getfile?taskID=" ++ show taskid ++ "&name=" ++ logFile file ++ "&offset=" ++ maybe "-6000" show moffset
 
 logFile :: LogFile -> String
 logFile RootLog = "root.log"
 logFile BuildLog = "build.log"
 logFile HWInfo = "hw_info.log"
 
-buildlogSize :: Bool -> Bool -> Bool -> Maybe String -> String -> TaskResult
-             -> IO ()
-buildlogSize _debug tail' hwinfo mgrep hub task = do
+buildlogSize :: Bool -> TimeZone -> Bool -> LogFile -> Maybe String -> String
+             -> TaskResult -> IO ()
+buildlogSize debug tz tail' logfile mgrep hub task = do
   murl <- findOutputURL hub task
+  when debug $ print murl
   whenJust murl $ \ url -> do
     let buildlog = url +/+ logFile BuildLog
+    when debug $ print buildlog
     exists <- httpExists' buildlog
     if exists
       then do
-      putStr $ buildlog ++ " "
-      msize <- httpFileSize' buildlog
-      case msize of
-        Nothing -> putChar '\n'
-        Just size -> do
-          fprintLn ("(" % commas % "kB)") (size `div` 1000)
-          -- FIXME check if short build.log ends with srpm
-          file <-
-            if hwinfo
-            then do
-              putStrLn $ url +/+ logFile HWInfo
-              return HWInfo
-            else
-              -- for buildroot failure build.log could be ~3082 bytes
-              if size < 4000
-              then do
-                putStrLn $ url +/+ logFile RootLog
-                return RootLog
-              else return BuildLog
-          when (tail' || hwinfo || isJust mgrep) $ displayLog url file
+      (msize,mtime) <- httpFileSizeTime' buildlog
+      whenJust msize $ \size -> do
+        putStr buildlog
+        fprint (" (" % commas % "kB)") (size `div` 1000)
+        when (taskState task == TaskOpen) $
+          whenJust mtime $ \time ->
+          putStr $ " (" ++ compactZonedTime tz time ++ ")"
+        putChar '\n'
+        let file =
+              if logfile == BuildLog && size < 4000
+                then RootLog
+                else logfile
+        unless (file == BuildLog) $
+          putStrLn $ url +/+ logFile file
+        when (tail' || logfile /= BuildLog || isJust mgrep) $ do
+          displayLog size url file
+          (putStrLn . compactTaskResult hub tz) task
       else do
       let rootlog = url +/+ logFile RootLog
       whenM (httpExists' rootlog) $
         putStrLn rootlog
   where
-    displayLog :: String -> LogFile -> IO ()
-    displayLog url file = do
+    displayLog :: Integer -> String -> LogFile -> IO ()
+    displayLog buildlogsize url file = do
       let logurl =
             case file of
-              BuildLog -> tailLogUrl hub (taskId task) file
+              BuildLog -> tailLogUrl hub (taskId task) file Nothing
               _ -> url +/+  logFile file
       req <- parseRequest logurl
-      resp <- httpLBS req
-      let out = U.toString $ getResponseBody resp
-          ls = lines out
-      putStrLn ""
-      let output
-            | file == RootLog =
-              let excluded = ["Executing command:",
-                              "Child return code was: 0",
-                              "child environment: None",
-                              "ensuring that dir exists:",
-                              "touching file:",
-                              "creating dir:",
-                              "kill orphans"]
-              in
-                map (dropPrefix "DEBUG ") $ takeEnd 30 $
-                filter (\l -> not (any (`isInfixOf` l) excluded)) ls
-            | last ls == "Child return code was: 0" = ls
-            | otherwise =
-                case breakOnEnd ["Child return code was: 1"] ls of
-                  ([],ls') -> ls'
-                  (ls',_) -> ls'
-      putStr $ unlines $
-        case mgrep of
-          Nothing -> output
-          Just needle ->
-            filter (match needle) ls
-      putStrLn $ "\n" ++ logurl
-      where
-        match :: String -> String -> Bool
-        match "" _ = error' "empty grep string not allowed"
-        match _ "" = False
-        match ('^':needle) ls =
-          if last needle == '$'
-          then needle == ls
-          else needle `isPrefixOf` ls
-        match needle ls =
-          if last needle == '$'
-          then needle `isSuffixOf` ls
-          else needle `isInfixOf` ls
+      resp <- httpLBS $ addRequestHeader hAccept "text/plain" req
+      let ls = lines . U.toString $ getResponseBody resp
+      -- detect dynbr failure (build.log ~ 7400B):
+      -- Wrote: /builddir/build/SRPMS/bustle-0.13.0-1.fc42.buildreqs.nosrc.rpm
+      -- Child return code was: 11
+      -- Dynamic buildrequires detected
+      -- Going to install missing buildrequires. See root.log for details.
+      if file == BuildLog && buildlogsize < 8000 &&
+        lastMay ls == Just "Going to install missing buildrequires. See root.log for details."
+        then displayLog buildlogsize url RootLog
+        else do
+        putStrLn ""
+        let output
+              | file == RootLog =
+                let excluded = ["Executing command:",
+                                "Child return code was: 0",
+                                "child environment: None",
+                                "ensuring that dir exists:",
+                                "touching file:",
+                                "creating dir:",
+                                "kill orphans"]
+                in
+                  map (dropPrefix "DEBUG ") $ takeEnd 30 $
+                  filter (\l -> not (any (`isInfixOf` l) excluded)) ls
+              | lastMay ls == Just "Child return code was: 0" = ls
+              | otherwise =
+                  case breakOnEnd ["Child return code was: 1"] ls of
+                    ([],ls') -> ls'
+                    (ls',_) -> ls'
+        putStr $ unlines $
+          case mgrep of
+            Nothing -> output
+            Just needle ->
+              filter (match needle) ls
+        putStrLn $ '\n' : logurl
+        when (file == BuildLog) $
+          putStrLn $ url +/+ logFile file ++ "\n"
+        where
+          match :: String -> String -> Bool
+          match "" _ = error' "empty grep string not allowed"
+          match _ "" = False
+          match ('^':needle) ls =
+            if lastMay needle == Just '$'
+            then needle == ls
+            else needle `isPrefixOf` ls
+          match needle ls =
+            if lastMay needle == Just '$'
+            then needle `isSuffixOf` ls
+            else needle `isInfixOf` ls
 
 -- FIXME turn into a type?
 kojiMethods :: [String]
