@@ -2,14 +2,17 @@ module Utils (
   kojiTaskRequestNVR,
   kojiGetBuildID',
   kojiGetBuild',
+  kojiTaskArch,
   showValue,
   buildOutputURL,
-  hubToPkgsURL
+  hubToPkgsURL,
+  buildlogUrlfromTaskId,
+  worktaskDirUrlfromTaskID
   )
 where
 
-import Data.List.Extra (dropSuffix, isInfixOf, isPrefixOf, isSuffixOf, replace,
-                        takeWhileEnd)
+import Data.List.Extra (dropPrefix, dropSuffix, isInfixOf, isPrefixOf,
+                        isSuffixOf, replace, takeEnd, takeWhileEnd)
 import Data.RPM (dropArch)
 import Data.RPM.NVR
 import Data.RPM.NVRA
@@ -36,7 +39,7 @@ kojiTaskRequestNVR task =
                    Left $
                    -- fedora-ci_fc87dcd7db694cdfac703cf5af128a72_c1f53c859616fdbf40811058b60bebe2930e3271_0;forks:petersen:rpms:grub2.f38.src.rpm
                    if "fedora-ci_" `isPrefixOf` base
-                   then takeWhileEnd (/= ':') $ tail $ dropWhile (/= ';') base
+                   then takeWhileEnd (/= ':') $ dropPrefix ";" $ dropWhile (/= ';') base
                    else base
     _ -> error' "could determine package from build request"
 
@@ -78,3 +81,22 @@ hubToPkgsURL url =
       if "kojihub" `isSuffixOf` url
       then replace "kojihub" "kojifiles" url +/+ "packages"
       else error' $ "use --files-url to specify kojifiles url for " ++ url
+
+buildlogUrlfromTaskId :: Int -> String
+buildlogUrlfromTaskId tid =
+  worktaskDirUrlfromTaskID tid +/+ "build.log"
+
+worktaskDirUrlfromTaskID :: Int -> String
+worktaskDirUrlfromTaskID tid =
+  "https://kojipkgs.fedoraproject.org/work/tasks" +/+ lastFew +/+ taskid
+  where
+    taskid = show tid
+
+    lastFew =
+      let few = dropWhile (== '0') $ takeEnd 4 taskid in
+        if null few then "0" else few
+
+kojiTaskArch :: String -> String
+kojiTaskArch "i686" = "i386"
+kojiTaskArch "armv7hl" = "armhfp"
+kojiTaskArch a = a
